"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const schema_command_1 = tslib_1.__importDefault(require("../../lib/schema-command"));
const color_1 = require("../../lib/color");
const core_1 = require("@oclif/core");
class DiffSchemaCommand extends schema_command_1.default {
    async run() {
        var _a;
        const [source, target] = this.parseTarget();
        const fps = this.gatherRelativeFSLFilePaths();
        const files = this.read(fps);
        const { url, secret } = await this.fetchsetup();
        let version = undefined;
        let status = "";
        const diffKind = ((_a = this.flags) === null || _a === void 0 ? void 0 : _a.text) ? "textual" : "semantic";
        try {
            const params = new URLSearchParams({
                ...((0, color_1.hasColor)() ? { color: (0, color_1.colorParam)() } : {}),
                ...(target === "staged" ? { diff: diffKind } : {}),
            });
            const res = await fetch(new URL(`/schema/1/staged/status?${params}`, url), {
                method: "GET",
                headers: { AUTHORIZATION: `Bearer ${secret}` },
                // https://github.com/nodejs/node/issues/46221
                // https://github.com/microsoft/TypeScript-DOM-lib-generator/issues/1483
                // @ts-expect-error-next-line
                duplex: "half",
            });
            const statusJson = await res.json();
            if (statusJson.error) {
                this.error(statusJson.error.message);
            }
            version = statusJson.version;
            status = statusJson.status;
            if (target === "staged") {
                this.log(`Differences from the ${(0, color_1.bold)()}remote, active${(0, color_1.reset)()} schema to the ${(0, color_1.bold)()}remote, staged${(0, color_1.reset)()} schema:`);
                if (status === "none") {
                    this.log("There is no staged schema present.");
                }
                else {
                    this.log(statusJson.diff ? statusJson.diff : "No schema differences");
                }
            }
            else {
                const params = new URLSearchParams({
                    ...((0, color_1.hasColor)() ? { color: (0, color_1.colorParam)() } : {}),
                    staged: source === "staged" ? "true" : "false",
                    ...(version !== undefined ? { version } : { force: "true" }),
                    diff: diffKind,
                });
                const res = await fetch(new URL(`/schema/1/validate?${params}`, url), {
                    method: "POST",
                    headers: { AUTHORIZATION: `Bearer ${secret}` },
                    body: this.body(files),
                    // @ts-expect-error-next-line
                    duplex: "half",
                });
                const json = await res.json();
                if (json.error) {
                    this.error(json.error.message);
                }
                if (status === "none") {
                    this.log(`Differences from the ${(0, color_1.bold)()}remote${(0, color_1.reset)()} schema to the ${(0, color_1.bold)()}local${(0, color_1.reset)()} schema:`);
                }
                else if (source === "active") {
                    this.log(`Differences from the ${(0, color_1.bold)()}remote, active${(0, color_1.reset)()} schema to the ${(0, color_1.bold)()}local${(0, color_1.reset)()} schema:`);
                }
                else {
                    this.log(`Differences from the ${(0, color_1.bold)()}remote, staged${(0, color_1.reset)()} schema to the ${(0, color_1.bold)()}local${(0, color_1.reset)()} schema:`);
                }
                this.log(json.diff ? json.diff : "No schema differences");
            }
        }
        catch (err) {
            this.error(err);
        }
    }
    parseTarget() {
        var _a, _b, _c, _d, _e, _f;
        if (!((_a = this.flags) === null || _a === void 0 ? void 0 : _a.active) && !((_b = this.flags) === null || _b === void 0 ? void 0 : _b.staged)) {
            return ["staged", "local"];
        }
        if (((_c = this.flags) === null || _c === void 0 ? void 0 : _c.active) && ((_d = this.flags) === null || _d === void 0 ? void 0 : _d.staged)) {
            this.error("Cannot specify both --active and --staged");
        }
        if ((_e = this.flags) === null || _e === void 0 ? void 0 : _e.active) {
            return ["active", "local"];
        }
        else if ((_f = this.flags) === null || _f === void 0 ? void 0 : _f.staged) {
            return ["active", "staged"];
        }
        else {
            // NB: `this.error` quits the program, so return `any` to make typescript happy.
            this.error("Invalid target. Expected: active or staged");
            return [];
        }
    }
}
DiffSchemaCommand.flags = {
    ...schema_command_1.default.flags,
    text: core_1.Flags.boolean({
        description: "Display the text diff instead of the semantic diff.",
        default: false,
    }),
    active: core_1.Flags.boolean({
        description: "Show the diff against the active schema instead of the staged schema.",
        default: false,
    }),
    staged: core_1.Flags.boolean({
        description: "Show the diff between the active and staged schema, instead of the local schema.",
        default: false,
    }),
};
DiffSchemaCommand.description = "Print the diff between local schema and staged remote schema.";
DiffSchemaCommand.examples = [
    "$ fauna schema diff",
    "$ fauna schema diff --dir schemas/myschema",
    "$ fauna schema diff staged",
    "$ fauna schema diff staged --text",
];
exports.default = DiffSchemaCommand;
