"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const prompts_1 = require("@inquirer/prompts");
const schema_command_1 = tslib_1.__importDefault(require("../../lib/schema-command"));
const core_1 = require("@oclif/core");
const color_1 = require("../../lib/color");
class PushSchemaCommand extends schema_command_1.default {
    async run() {
        var _a, _b, _c, _d, _e, _f, _g;
        const fps = this.gatherRelativeFSLFilePaths();
        const files = this.read(fps);
        try {
            const { url, secret } = await this.fetchsetup();
            const statusres = await fetch(new URL(`/schema/1/staged/status`, url), {
                method: "GET",
                headers: { AUTHORIZATION: `Bearer ${secret}` },
                // https://github.com/nodejs/node/issues/46221
                // https://github.com/microsoft/TypeScript-DOM-lib-generator/issues/1483
                // @ts-expect-error-next-line
                duplex: "half",
            });
            const statusjson = await statusres.json();
            if (statusjson.error) {
                this.error(statusjson.error.message);
            }
            if (statusjson.status !== "none" && ((_a = this.flags) === null || _a === void 0 ? void 0 : _a.active)) {
                this.error("Cannot skip a staged push while there is a staged schema.\n" +
                    "Use `fauna schema status` to check the staged schema.");
            }
            // Double negatives are confusing.
            const isStagedPush = !((_b = this.flags) === null || _b === void 0 ? void 0 : _b.active);
            if ((_c = this.flags) === null || _c === void 0 ? void 0 : _c["no-input"]) {
                const params = new URLSearchParams({
                    force: "true", // Just push.
                    staged: isStagedPush ? "true" : "false",
                });
                // This is how MDN says to do it for some reason.
                const path = new URL(`/schema/1/update?${params}`, url);
                const res = await fetch(path, {
                    method: "POST",
                    headers: { AUTHORIZATION: `Bearer ${secret}` },
                    body: this.body(files),
                    // @ts-expect-error-next-line
                    duplex: "half",
                });
                const json = await res.json();
                if (json.error) {
                    this.error((_e = (_d = json.error) === null || _d === void 0 ? void 0 : _d.message) !== null && _e !== void 0 ? _e : json.error);
                }
            }
            else {
                // Confirm diff, then push it. `force` is set on `validate` so we don't
                // need to pass the last known schema version through.
                const params = new URLSearchParams({
                    ...((0, color_1.hasColor)() ? { color: (0, color_1.colorParam)() } : {}),
                    force: "true",
                    diff: "summary",
                });
                const path = new URL(`/schema/1/validate?${params}`, url);
                const res = await fetch(path, {
                    method: "POST",
                    headers: { AUTHORIZATION: `Bearer ${secret}` },
                    body: this.body(files),
                    // @ts-expect-error-next-line
                    duplex: "half",
                });
                const json = await res.json();
                if (json.error) {
                    this.error((_g = (_f = json.error) === null || _f === void 0 ? void 0 : _f.message) !== null && _g !== void 0 ? _g : json.error);
                }
                let message = isStagedPush
                    ? "Stage the above changes?"
                    : "Push the above changes?";
                if (json.diff) {
                    this.log("Proposed diff:\n");
                    this.log(json.diff);
                }
                else {
                    this.log("No logical changes.");
                    message = "Stage the file contents anyway?";
                }
                if (!isStagedPush) {
                    this.log("Note: Any modified indexes will be temporarily unavailable while building.");
                }
                this.log("(use `fauna schema diff` to show the complete diff)");
                const confirmed = await (0, prompts_1.confirm)({
                    message,
                    default: false,
                });
                if (confirmed) {
                    const params = new URLSearchParams({
                        version: json.version,
                        staged: isStagedPush ? "true" : "false",
                    });
                    const path = new URL(`/schema/1/update?${params}`, url);
                    const res = await fetch(path, {
                        method: "POST",
                        headers: { AUTHORIZATION: `Bearer ${secret}` },
                        body: this.body(files),
                        // @ts-expect-error-next-line
                        duplex: "half",
                    });
                    const json0 = await res.json();
                    if (json0.error) {
                        this.error(json0.error.message);
                    }
                }
                else {
                    this.log("Push cancelled");
                }
            }
        }
        catch (err) {
            this.error(err);
        }
    }
}
PushSchemaCommand.flags = {
    ...schema_command_1.default.flags,
    active: core_1.Flags.boolean({
        description: "Skip staging the schema and make the schema active immediately. This will cause indexes to be temporarily unavailable.",
        default: false,
    }),
};
PushSchemaCommand.description = "Push the current project's .fsl files to Fauna.";
PushSchemaCommand.examples = [
    "$ fauna schema push",
    "$ fauna schema push --dir schemas/myschema",
    "$ fauna schema push --active",
];
exports.default = PushSchemaCommand;
