export default FaunaCommand;
/**
 * This is the base class for all fauna-shell commands.
 */
declare class FaunaCommand extends Command {
    clients: {};
    /**
     * This is used to determine if the command should log the connection info.
     * We currently want to avoid doing this for eval since it can lead to the
     * response not being JSON parseable.
     */
    outputConnectionInfo: boolean;
    /**
     * During init we parse the flags and arguments and assign them
     * to the `flags` and `args` member variables.
     *
     * We call `this.parse(this.constructor)` because we need to load
     * flags and args for the command being run in the CLI.
     * In this way we parse the flags and args defined in that command,
     * plus the ones defined here. A command then needs to define its flags
     * as follows, if it wants to inherit the flags defined in FaunaCommand:
     *
     * CreateKeyCommand.flags = {
     *        ...FaunaCommand.flags
     * }
     *
     */
    init(): Promise<void>;
    flags: (import("@oclif/core/lib/interfaces/parser").FlagOutput & {
        json: boolean | undefined;
    }) | undefined;
    args: import("@oclif/core/lib/interfaces/parser").ArgOutput | undefined;
    shellConfig: ShellConfig | undefined;
    success(msg: any): void;
    error(message: any): void;
    _getHeaders(): {
        "X-Fauna-Source": string;
    };
    /**
     * !!! use getClient instead
     * Runs the function in the context of a database connection.
     *
     * @param {function} f       - The function to run
     * @param {string}   dbScope - The database in which the function will be executed.
     * @param {string}   role    - The user role with which the function will be executed.
     */
    withClient(f: Function, dbScope: string, role: string): Promise<any>;
    mapConnectionError({ err, connectionOptions }: {
        err: any;
        connectionOptions: any;
    }): void;
    getClient({ dbScope, role, version }?: {
        dbScope: any;
        role: any;
        version: any;
    }): Promise<any>;
    ensureDbScopeClient({ scope, version }: {
        scope: any;
        version: any;
    }): Promise<any>;
    /**
     * Runs the provided query, while logging a message before running it.
     * Calls the success callback on success, or the failure one otherwise.
     *
     * @param {query}    queryExpr - The Query to execute.
     * @param {string}   logMsg    - The message to display before executing the query.
     * @param {function} success   - On success callback.
     * @param {function} failure   - On error callback.
     */
    query(queryExpr: query, logMsg: string, success: Function, failure: Function): Promise<any>;
    dbExists(dbName: any, callback: any): Promise<any>;
}
declare namespace FaunaCommand {
    let flags: {
        domain: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        scheme: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        port: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        url: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        timeout: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        secret: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        endpoint: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        environment: import("@oclif/core/lib/interfaces").OptionFlag<string | undefined, import("@oclif/core/lib/interfaces").CustomOptions>;
        color: import("@oclif/core/lib/interfaces").BooleanFlag<boolean>;
        "no-input": import("@oclif/core/lib/interfaces").BooleanFlag<boolean>;
    };
}
import { Command } from "@oclif/core";
import { ShellConfig } from "./config";
